/*
 * Copyright © 2022 Soren Stoutner <soren@stoutner.com>.
 *
 * This file is part of Privacy Browser PC <https://www.stoutner.com/privacy-browser-pc>.
 *
 * Privacy Browser PC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Privacy Browser PC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Privacy Browser PC.  If not, see <http://www.gnu.org/licenses/>.
 */

// Application headers
#include "BrowserWindow.h"
#include "MainView.h"
#include "MouseEventFilter.h"
#include "Settings.h"
#include "ui_MainView.h"
#include "UserAgentHelper.h"

// Qt framework headers.
#include <QWebEngineProfile>

MainView::MainView(QWidget *parent) : QWidget(parent)
{
    // Instantiate the mainview UI.
    Ui::MainView mainViewUi;

    // Setup the UI.
    mainViewUi.setupUi(this);

    // Get handles for the views.
    backButtonPointer = mainViewUi.backButton;
    forwardButtonPointer = mainViewUi.forwardButton;
    QPushButton *refreshButtonPointer = mainViewUi.refreshButton;
    QPushButton *homeButtonPointer = mainViewUi.homeButton;
    urlLineEditPointer = mainViewUi.urlLineEdit;
    javaScriptButtonPointer = mainViewUi.javaScript;
    webEngineViewPointer = mainViewUi.webEngineView;

    // Get handles for the aspects of the WebEngine.
    QWebEnginePage *webEnginePagePointer = webEngineViewPointer->page();
    webEngineHistoryPointer = webEnginePagePointer->history();
    webEngineProfilePointer = webEnginePagePointer->profile();
    webEngineSettingsPointer = webEngineViewPointer->settings();

    // Update the webengine view from the URL line edit.
    connect(urlLineEditPointer, SIGNAL(returnKeyPressed(const QString)), this, SLOT(loadUrlFromTextBox(const QString)));

    // Update the URL line edit form the webengine view.
    connect(webEngineViewPointer, SIGNAL(loadStarted()), this, SLOT(updateInterface()));
    connect(webEngineViewPointer, SIGNAL(loadProgress(int)), this, SLOT(updateInterface()));
    connect(webEngineViewPointer, SIGNAL(loadFinished(bool)), this, SLOT(updateInterface()));

    // Setup the URL bar buttons.
    connect(backButtonPointer, SIGNAL(clicked()), webEngineViewPointer, SLOT(back()));
    connect(forwardButtonPointer, SIGNAL(clicked()), webEngineViewPointer, SLOT(forward()));
    connect(refreshButtonPointer, SIGNAL(clicked()), webEngineViewPointer, SLOT(reload()));
    connect(homeButtonPointer, SIGNAL(clicked()), this, SLOT(goHome()));
    connect(javaScriptButtonPointer, SIGNAL(clicked()), this, SLOT(toggleJavaScript()));

    // Instantiate the mouse event pointer.
    MouseEventFilter *mouseEventFilterPointer = new MouseEventFilter(webEngineViewPointer);

    // Install the mouse event filter.
    qApp->installEventFilter(mouseEventFilterPointer);

    // Listen for hovered link URLs.
    connect(webEnginePagePointer, SIGNAL(linkHovered(QString)), this, SLOT(pageLinkHovered(QString)));

    // Disable the cache.
    webEngineProfilePointer->setHttpCacheType(QWebEngineProfile::NoCache);

    // Don't allow JavaScript to open windows.
    webEngineSettingsPointer->setAttribute(QWebEngineSettings::JavascriptCanOpenWindows, false);

    // Set the zoom factor.
    webEngineViewPointer->setZoomFactor(Settings::zoomFactor());

    // Apply the application settings.
    applyApplicationSettings();

    // Apply the domain settings.  `false` does not reload the website.
    applyDomainSettings(false);

    // Set the focus on the WebEngine view.
    webEngineViewPointer->setFocus();

    // Get the arguments.
    QStringList argumentsStringList = qApp->arguments();

    // Check to see if the arguments lists contains a URL.
    if (argumentsStringList.size() > 1)
    {
        // Load the URL from the arguments list.
        webEngineViewPointer->setUrl(QUrl::fromUserInput(argumentsStringList.at(1)));
    }
    else
    {
        // Load the homepage.
        goHome();
    }
}

void MainView::applyApplicationSettings() const
{
    // TODO.
}

// This exists as a separate function from `applyDomainSettings()` so it can be listed as a slot and function without the need for a boolean argument.
void MainView::applyDomainSettingsAndReload() const
{
    // Apply the domain setings.  `true` reloads the website.
    applyDomainSettings(true);
}

void MainView::applyDomainSettings(bool reloadWebsite) const
{
    // Set the JavaScript status.
    webEngineSettingsPointer->setAttribute(QWebEngineSettings::JavascriptEnabled, Settings::javaScript());

    // Update the JavaScript button.
    if (Settings::javaScript())
    {
        javaScriptButtonPointer->setIcon(QIcon(":/icons/javascript-warning"));
    }
    else
    {
        javaScriptButtonPointer->setIcon(QIcon(":/icons/privacy-mode"));
    }

    // Apply the user agent.
    webEngineProfilePointer->setHttpUserAgent(UserAgentHelper::getUserAgent(Settings::userAgent()));

    // Reload the website if requested.
    if (reloadWebsite)
    {
        webEngineViewPointer->reload();
    }
}

void MainView::goHome() const
{
    // Load the homepage.
    webEngineViewPointer->setUrl(QUrl::fromUserInput(Settings::homepage().toString()));
}

void MainView::loadUrlFromTextBox(QString urlFromUser) const
{
    // Remove the focus from the URL line edit.
    urlLineEditPointer->clearFocus();

    // Check if the URL does not start with a valid protocol.
    if (!urlFromUser.startsWith("http") && !urlFromUser.startsWith("file://"))
    {
        // Add `https://` to the beginning of the URL.
        urlFromUser = "https://" + urlFromUser;
    }

    // Load the URL.
    webEngineViewPointer->setUrl(QUrl::fromUserInput(urlFromUser));
}

void MainView::pageLinkHovered(const QString &linkUrl) const
{
    // Emit a signal so that the browser window can update the status bar.
    emit linkHovered(linkUrl);
}

void MainView::toggleJavaScript() const
{
    // Toggle JavaScript.
    webEngineSettingsPointer->setAttribute(QWebEngineSettings::JavascriptEnabled, !webEngineSettingsPointer->testAttribute(QWebEngineSettings::JavascriptEnabled));

    // Update the JavaScript button.
    if (webEngineSettingsPointer->testAttribute(QWebEngineSettings::JavascriptEnabled))
    {
        javaScriptButtonPointer->setIcon(QIcon(":/icons/javascript-warning"));
    }
    else
    {
        javaScriptButtonPointer->setIcon(QIcon(":/icons/privacy-mode"));
    }

    // Reload the website.
    webEngineViewPointer->reload();
}

void MainView::updateInterface() const
{
    // Update the URL line edit if it does not have focus.
    if (!urlLineEditPointer->hasFocus())
    {
        // Update the URL line edit.
        urlLineEditPointer->setText(webEngineViewPointer->url().toString());
    }

    // Update the status of the forward and back buttons.
    backButtonPointer->setEnabled(webEngineHistoryPointer->canGoBack());
    forwardButtonPointer->setEnabled(webEngineHistoryPointer->canGoForward());

    // Reapply the zoom factor.  This is a bug in QWebEngineView that resets the zoom with every load.  Hopefully it will be fixed in Qt6.  <https://bugreports.qt.io/browse/QTBUG-51992>
    webEngineViewPointer->setZoomFactor(Settings::zoomFactor());
}
